/*************************************************
* CTR Mode Source File                           *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/ctr.h>

namespace Botan {

/*************************************************
* CTR Constructor                                *
*************************************************/
CTR::CTR(const std::string& cipher_name, const SymmetricKey& key,
         const BlockCipherModeIV& iv) :
   BlockCipherMode(cipher_name, "CTR", key, iv)
   {
   cipher->encrypt(state, buffer);
   }

/*************************************************
* CTR Encryption/Decryption                      *
*************************************************/
void CTR::write(const byte input[], u32bit length)
   {
   while(length)
      {
      u32bit copied = std::min(BLOCK_SIZE - position, length);
      xor_buf(buffer + position, input, copied);
      send(buffer + position, copied);
      input += copied;
      length -= copied;
      position += copied;
      if(position == BLOCK_SIZE)
         {
         for(s32bit j = BLOCK_SIZE - 1; j >= 0; j--)
            if(++state[j])
               break;
         cipher->encrypt(state, buffer);
         position = 0;
         }
      }
   }

}
