/*************************************************
* MD2 Source File                                *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/md2.h>

namespace Botan {

/*************************************************
* MD2 Hash                                       *
*************************************************/
void MD2::hash(const byte input[])
   {
   X.copy(16, input, HASH_BLOCK_SIZE);
   xor_buf(X + 32, X, X + 16, HASH_BLOCK_SIZE);
   byte T = 0;
   for(u32bit j = 0; j != 18; j++)
      {
      for(u32bit k = 0; k != 48; k += 8)
         {
         T = X[k  ] ^= SBOX[T]; T = X[k+1] ^= SBOX[T];
         T = X[k+2] ^= SBOX[T]; T = X[k+3] ^= SBOX[T];
         T = X[k+4] ^= SBOX[T]; T = X[k+5] ^= SBOX[T];
         T = X[k+6] ^= SBOX[T]; T = X[k+7] ^= SBOX[T];
         }
      T += j;
      }
   T = checksum[15];
   for(u32bit j = 0; j != HASH_BLOCK_SIZE; j++)
      T = checksum[j] ^= SBOX[input[j] ^ T];
   }

/*************************************************
* Finalize a MD2 Hash                            *
*************************************************/
void MD2::final_result(byte output[])
   {
   for(u32bit j = position; j != HASH_BLOCK_SIZE; j++)
      buffer[j] = (byte)(HASH_BLOCK_SIZE - position);
   hash(buffer);
   hash(checksum);
   copy_mem(output, X.ptr(), OUTPUT_LENGTH);
   clear();
   }

/*************************************************
* Clear memory of sensitive data                 *
*************************************************/
void MD2::clear() throw()
   {
   X.clear();
   checksum.clear();
   buffer.clear();
   position = 0;
   }

}
