/*************************************************
* Block Cipher Mode Base Source File             *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/modebase.h>
#include <botan/lookup.h>

namespace Botan {

/*************************************************
* Block Cipher Mode Constructor                  *
*************************************************/
BlockCipherMode::BlockCipherMode(const std::string& cipher_name,
                                 const std::string& cipher_mode_name,
                                 const SymmetricKey& key,
                                 const BlockCipherModeIV& iv,
                                 u32bit buf_size_mult) :
   BLOCK_SIZE(block_size_of(cipher_name)),
   BUFFER_SIZE(buf_size_mult * BLOCK_SIZE),
   mode_name(cipher_mode_name),
   cipher(get_block_cipher(cipher_name)),
   buffer(BUFFER_SIZE)
   {
   if(iv.length())
      state.create(BLOCK_SIZE);
   reset_key(key);
   reset_iv(iv);
   position = 0;
   }

/*************************************************
* Return the name of this type                   *
*************************************************/
std::string BlockCipherMode::name() const
   {
   return mode_name + "(" + cipher->name() + ")";
   }

/*************************************************
* Set the IV                                     *
*************************************************/
void BlockCipherMode::reset_iv(const BlockCipherModeIV& new_iv)
   {
   if(new_iv.length() != state.size())
      throw Invalid_IV_Length(name(), new_iv.length());
   state.copy(new_iv, new_iv.length());
   buffer.clear();
   position = 0;
   }

/*************************************************
* Set the Key                                    *
*************************************************/
void BlockCipherMode::reset_key(const SymmetricKey& key)
   {
   cipher->set_key(key);
   }

}
