/*************************************************
* MP Multiplication Source File                  *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/mp_core.h>
#include <botan/util.h>

namespace Botan {

namespace {

/*************************************************
* Simple O(N^2) Multiplication                   *
*************************************************/
void simple_multiply(word* const z,
                     const word* x, u32bit x_size,
                     const word* y, u32bit y_size)
   {
   for(u32bit j = 0; j != x_size; j++)
       {
      word carry = 0, x_j = x[j];
      for(u32bit k = 0; k != y_size; k++)
         {
         dword product = (dword)z[j+k] + (dword)x_j * y[k] + carry;
         z[j+k] = MP_LOW_WORD(product);
         carry = MP_HIGH_WORD(product);
         }
      z[j+y_size] = carry;
      }
   }

}

/*************************************************
* MP Multiplication                              *
*************************************************/
void bigint_mul3(word* const z, word* const, u32bit,
                 const word* x, u32bit x_size, u32bit x_sw,
                 const word* y, u32bit y_size, u32bit y_sw)
   {
   if(x_size < 4 || y_size < 4)
      simple_multiply(z, x, x_sw, y, y_sw);
   else if(x_sw <= 4 && y_sw <= 4 && x_size >= 4 && y_size >= 4)
      bigint_comba4(z, x, y);
   else if(x_sw <= 8 && y_sw <= 8 && x_size >= 8 && y_size >= 8)
      bigint_comba8(z, x, y);
   else
      simple_multiply(z, x, x_sw, y, y_sw);
   }

}
