/*************************************************
* Miscellaneous PK Functions Source File         *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/pk_misc.h>

namespace Botan {

/*************************************************
* Encode a message                               *
*************************************************/
SecureVector<byte> EME::encode(const byte msg[], u32bit msg_len,
                               u32bit key_bits) const
   {
   return pad(msg, msg_len, key_bits);
   }

/*************************************************
* Encode a message                               *
*************************************************/
SecureVector<byte> EME::encode(const SecureVector<byte>& msg,
                               u32bit key_bits) const
   {
   return pad(msg, msg.size(), key_bits);
   }

/*************************************************
* Decode a message                               *
*************************************************/
SecureVector<byte> EME::decode(const byte msg[], u32bit msg_len,
                               u32bit key_bits) const
   {
   return unpad(msg, msg_len, key_bits);
   }

/*************************************************
* Decode a message                               *
*************************************************/
SecureVector<byte> EME::decode(const SecureVector<byte>& msg,
                               u32bit key_bits) const
   {
   return unpad(msg, msg.size(), key_bits);
   }

/*************************************************
* Default signature decoding                     *
*************************************************/
bool EMSA::verify(const SecureVector<byte>& coded,
                  const SecureVector<byte>& raw,
                  u32bit key_len) throw()
   {
   try {
      return (coded == encoding_of(raw, key_len));
      }
   catch(Invalid_Argument)
      {
      return false;
      }
   }

/*************************************************
* Derive a key                                   *
*************************************************/
SymmetricKey KDF::derive_key(const byte in[], u32bit in_len,
                             u32bit key_len) const
   {
   return derive(in, in_len, key_len);
   }

/*************************************************
* Derive a key                                   *
*************************************************/
SymmetricKey KDF::derive_key(const SecureVector<byte>& in,
                             u32bit key_len) const
   {
   return derive(in, in.size(), key_len);
   }

}
